@@ -18,7 +18,7 @@
 */
 
 static char *electricsheep_c_id =
-"@(#) $Id: electricsheep.c,v 1.45 2005/06/06 21:51:32 spotspot Exp $";
+"@(#) $Id: electricsheep.c,v 1.56 2005/12/13 06:55:28 spotspot Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -77,7 +77,7 @@ int init_delay = 600;
 int init_delay_list = 60;
 int list_freshness = 600;
 
-// ick xxx
+/* ick but ok since only 5 digits per field in sheep mpg filenames */
 #define max_anims 100000
 
 #define copy_buffer_size 64000
@@ -112,7 +112,6 @@ char *dream_server = "v2d6.sheepserver.net";
 #define max_cp_size 300000
 #define MAXBUF (5*MAXPATHLEN)
 
-
 char *play_prog = "mpeg2dec_onroot";
 char *nick_name = "";
 char *url_name = "";
@@ -290,11 +289,11 @@ mysystem2(char *cmd, char *msg) {
 }
 
 void timestamp() {
-#if 0
   time_t t;
-  time(&t);
-  puts(ctime(&t));
-#endif
+  if (debug) {
+      time(&t);
+      printf("time %s", ctime(&t));
+  }
 }
 
 static void
@@ -361,7 +360,9 @@ update_cached_anims(int match_gen) {
 	    if (0 == strncmp("gen", e->d_name, 3)) {
 		sscanf(e->d_name, "gen%d", &old_gen);
 		checked_gen = 1;
-		if (old_gen != generation) {
+		if (old_gen != generation && generation != -1) {
+		    if (debug)
+			printf("old_gen=%d generation=%d\n", old_gen, generation);
 		    write_gen = 1;
 		    clear_gen = 1;
 		}
@@ -380,7 +381,7 @@ update_cached_anims(int match_gen) {
 		}
 	    }
 	    if (i == nserver_anims && nserver_anims) {
-		sprintf(fbuf, "%s%s", leave_prefix, e->d_name);
+		snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
 		if (debug)
 		    printf("removing marker %s nserver_anims=%d\n",
 			   fbuf, nserver_anims);
@@ -392,7 +393,7 @@ update_cached_anims(int match_gen) {
 	}
  
 	if (!filename_is_mpg(e->d_name)) continue;
-	sprintf(fbuf, "%s%s", leave_prefix, e->d_name);
+	snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
 
 	if (4 != sscanf(e->d_name, "%d=%d=%d=%d.mpg",
 			&an->generation, &an->id, &an->first, &an->last)) {
@@ -431,19 +432,21 @@ update_cached_anims(int match_gen) {
 	print_anims("ncached_anims", cached_anims, ncached_anims);
 
     if (match_gen && !checked_gen) write_gen = 1;
-	
+
     if (clear_gen) {
 	DIR *d = opendir(leave_prefix);
 	if (debug) printf("killing all sheep!\n");
+
 	if (!d) {
 	    perror(leave_prefix);
 	    cleanup_and_exit(1);
 	}
+
 	while (e = readdir(d)) {
 	    if ((0 == strncmp("gen", e->d_name, 3)) ||
 		filename_is_mpg(e->d_name) ||
 		filename_is_xxx(e->d_name)) {
-		sprintf(fbuf, "%s%s", leave_prefix, e->d_name);
+		snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
 		unlink(fbuf);
 	    }
 	}
@@ -452,7 +455,7 @@ update_cached_anims(int match_gen) {
 	
     if (write_gen) {
 	if (debug) printf("write gen %d.\n", generation);
-	sprintf(fbuf, "touch %sgen%d", leave_prefix, generation);
+	snprintf(fbuf, MAXBUF, "touch %sgen%d", leave_prefix, generation);
 	mysystem(fbuf, "touch gen");
     }
 }
@@ -464,13 +467,13 @@ irandom(int n) {
 
 void
 cached_file_name(char *buf, anim_t *an) {
-    sprintf(buf, "%s%05d=%05d=%05d=%05d.mpg",
+    snprintf(buf, MAXBUF, "%s%05d=%05d=%05d=%05d.mpg",
 	    leave_prefix, an->generation, an->id, an->first, an->last);
 }
 
 void
 deleted_file_name(char *buf, anim_t *an) {
-    sprintf(buf, "%s%05d=%05d=%05d=%05d.xxx",
+    snprintf(buf, MAXBUF, "%s%05d=%05d=%05d=%05d.xxx",
 	    leave_prefix, an->generation, an->id, an->first, an->last);
 }
 
@@ -478,35 +481,12 @@ void
 not_playing() {
     char pbuf[MAXBUF];
     if (!read_only) {
-	sprintf(pbuf, "echo none > %sid", leave_prefix);
+	snprintf(pbuf, MAXBUF, "echo none > %sid", leave_prefix);
 	mysystem(pbuf, "writing none to id file");
     }
 }    
 
-void
-default_background(char *more) {
-    char ob[MAXBUF];
-    char pbuf[MAXBUF];
-    char qbuf[MAXBUF];
 
-    if (nobg || (!on_root && !window_id)) return;
-    if (more)
-	sprintf(ob, "-merge -at 500,0 %s-%s.tif",
-		splash_prefix, more);
-    else
-	ob[0] = 0;
-
-    if (window_id)
-      sprintf(qbuf, "-windowid %s", window_id);
-    else
-      qbuf[0] = 0;
-
-    sprintf(pbuf, "xsetbg %s "
-	    "-border black -at 0,0 %s-splash-0.tif " 
-	    "-merge -center %s-splash-1.tif %s",
-	    qbuf, splash_prefix, splash_prefix, ob);
-    mysystem2(pbuf, "splash0");
-}
 
 #define max_plays 1000
 
@@ -566,6 +546,7 @@ do_display() {
     int i;
     int idx;
     int niters;
+    static patience=1;
 
     if (start_id >= 0) {
 	current_anim_id = start_id;
@@ -589,9 +570,11 @@ do_display() {
 	    if (!cached_anims[i].deleted) n++;
         }
 	if (0 == n) {
-	    if (debug) printf("nothing to play, sleeping.\n");
+	    if (patience)
+		fprintf(stderr, "please be patient while the "
+			"first sheep is downloaded...\n");
+	    patience = 0;
 	    not_playing();
-	    default_background(0);
 	    sleep(10);
 	    update_cached_anims(0);
 	    return;
@@ -627,7 +610,7 @@ do_display() {
 	    nloopplays++;
 	}
 
-	sprintf(fbuf, "%sid.tmp", leave_prefix);
+	snprintf(fbuf, MAXBUF, "%sid.tmp", leave_prefix);
 	idf = fopen(fbuf, "w");
 	if (NULL == idf) {
 	    perror(fbuf);
@@ -639,7 +622,7 @@ do_display() {
 		fprintf(idf, "%d\n", path_history[h]);
 	    }
 	fclose(idf);
-	sprintf(pbuf, "%sid", leave_prefix);
+	snprintf(pbuf, MAXBUF, "%sid", leave_prefix);
 	if (-1 == rename(fbuf, pbuf)) {
 	    fprintf(stderr, "rename %s %s\n", fbuf, pbuf);
 	    perror(pbuf);
@@ -651,7 +634,7 @@ do_display() {
 	}
 	path_history[0] = current_anim_id;
 
-	if (debug) {
+	if (debug > 1) {
 	    printf("history =");
 	    for (h = 0; h < history_size; h++) {
 	      if (-1 == path_history[h])
@@ -724,8 +707,8 @@ do_display() {
 			    break;
 		    }
 		    if (hh == h) {
-			if (debug) printf("succ x=%d id=%d h=%d\n",
-					  i, cached_anims[i].id, h);
+			if (debug > 1) printf("succ x=%d id=%d h=%d\n",
+					      i, cached_anims[i].id, h);
 			if (cached_anims[i].first == cached_anims[i].last)
 			    lsuccs[lnsuccs++] = i;
 			else
@@ -836,10 +819,10 @@ download_anim(int idx) {
     if (debug) printf("download %d id=%d\n", idx, server_anims[idx].id);
     
     cached_file_name(cfb, &server_anims[idx]);
-    sprintf(tfb, "%s.tmp", cfb);
+    snprintf(tfb, MAXBUF, "%s.tmp", cfb);
     strcpy(mpg_name, tfb);
     
-    sprintf(pbuf, "%s --silent --show-error --output %s %s",
+    snprintf(pbuf, MAXBUF, "%s --silent --show-error --output %s %s",
 	    curl_cmd, tfb, server_anims[idx].url);
 
     if (debug) printf("about to %s\n", pbuf);
@@ -883,7 +866,7 @@ get_control_points(char *buf, int buf_size) {
     char pbuf[MAXBUF];
     FILE *cp;
   
-    sprintf(pbuf, "%s --silent --show-error 'http://%s/cgi/get?"
+    snprintf(pbuf, MAXBUF, "%s --silent --show-error 'http://%s/cgi/get?"
 	    "n=%s&w=%s&v=%s&u=%s' | gunzip -c %s",
 	    curl_cmd, dream_server, nick_buf,
 	    url_buf, client_version, uniqueid, hide_stderr);
@@ -929,7 +912,7 @@ put_image(char *fname, int frame, int anim_id, int gen) {
 	cleanup_and_exit(1);
     }
 
-    sprintf(pbuf, "%s --silent --upload-file %s "
+    snprintf(pbuf, MAXBUF, "%s --silent --upload-file %s "
 	    "'http://%s/cgi/put?f=%d&id=%d&s=%ld&g=%d&v=%s&u=%s'",
 	    curl_cmd, fname, dream_server, frame, anim_id,
 	    (long)sbuf.st_size, gen, client_version, uniqueid);
@@ -1020,6 +1003,7 @@ void get_args(char *args, char *fname) {
 	cleanup_and_exit(1);
       }
       strncpy(args, beg+1, end-beg-1);
+      args[end-beg-1] = 0;
       break;
     }
   }
@@ -1035,6 +1019,8 @@ do_render() {
     FILE *fp;
     XML_Parser parser;
 
+    timestamp();
+
     get_control_points(anim_text, max_cp_size);
 
     if (0 == anim_text[0]) {
@@ -1086,10 +1072,9 @@ do_render() {
 
     {
 	char b[MAXBUF];
-	char strips[MAXBUF];
 	char args[MAXBUF];
 	get_args(args, cps_name);
-	sprintf(b, "nice -n %d env in=%s time=%d out=%s %s flam3-animate",
+	snprintf(b, MAXBUF, "nice -n %d env in=%s time=%d out=%s %s flam3-animate",
 		nice_level, cps_name, get_frame, jpg_name, args);
 	if (debug) printf("about to render %s\n", b);
 	if (mysystem(b, "render")) {
@@ -1204,6 +1189,7 @@ void update_server_anims() {
     char pbuf[MAXBUF];
     FILE *lf;
     int done;
+    XML_Parser parser;
 
     if (-1 == server_anims_timestamp)
       server_anims_timestamp = time(0);
@@ -1216,13 +1202,13 @@ void update_server_anims() {
       server_anims_timestamp = now;
     }
 	
-    XML_Parser parser = XML_ParserCreate(NULL);
+    parser = XML_ParserCreate(NULL);
     XML_SetElementHandler(parser, list_start_element, get_end_element);
     XML_SetCharacterDataHandler(parser, character_handler);
     in_message = 0;
     server_error = 0;
     
-    sprintf(pbuf, "%s --silent --show-error 'http://%s/cgi/list?v=%s&u=%s'"
+    snprintf(pbuf, MAXBUF, "%s --silent --show-error 'http://%s/cgi/list?v=%s&u=%s'"
 	    "| gunzip -c %s", curl_cmd, dream_server,
 	    client_version, uniqueid, hide_stderr);
 
@@ -1419,7 +1405,7 @@ make_display_process()  {
 	    int c = 0;
 	    char *wid;
 	    /* child */
-	    sprintf(fps, "%d", frame_rate);
+	    snprintf(fps, 14, "%d", frame_rate);
 	    if (use_mplayer) {
 	      int devnull = open("/dev/null", O_WRONLY);
 	      if (-1 == devnull) {
@@ -1482,7 +1468,8 @@ make_display_process()  {
 		cleanup_and_exit(1);
 	    }
 	    execvp(play_prog, argv);
-	    perror("exec play_prog");
+	    perror(play_prog);
+	    cleanup_and_exit(1);
 	} else {
 #ifdef HAVE_SETPROCTITLE
 	    setproctitle("display");
@@ -1520,23 +1507,6 @@ make_display_process()  {
     /* parent returns */
 }
 
-
-void
-logo_only() {
-  char pbuf[MAXBUF];
-  char qbuf[MAXBUF];
-  if (!on_root && !window_id) return;
-  if (window_id)
-    sprintf(qbuf, "-windowid %s", window_id);
-  else
-    qbuf[0] = 0;
-  
-  sprintf(pbuf, "xsetbg %s -at 0,0 %s-splash-1.tif",
-	  qbuf, splash_prefix);
-  mysystem2(pbuf, "logo");
-  while (1) sleep(60);
-}
-
 /* set the number of threads by reading /proc/cpuinfo */
 void
 auto_nthreads() {
@@ -1640,7 +1610,7 @@ char *help_string =
 "--root 0/1 (display on root window, default 0)\n"
 "-window-id id (display in an existing window, note single dash!)\n"
 "--debug 0/1 (prints additional info)\n"
-"--voting 0/1 (enables voting, default 0)\n"
+"--voting 0/1 (enables voting, default 1)\n"
 "--anim-only 0/1 (leave background untouched, default 0)\n"
 "--mplayer 0/1 (use mplayer instead of the built-in decoder, default 0)\n"
 "--zoom 0/1 (zoom to fullscreen, default 0)\n"
@@ -1655,8 +1625,8 @@ char *help_string =
 void
 flags_init(int *argc, char ***argv) {
     char *arg0 = (*argv)[0];
-    char *bracket_begin;
-    char *bracket_end;
+    char *bracket_begin = 0;
+    char *bracket_end = 0;
     while (*argc > 1 &&
 	   (*argv)[1][0] == '-') {
 	char *o = (*argv)[1];
@@ -1707,6 +1677,36 @@ flags_init(int *argc, char ***argv) {
     parse_bracket(bracket_begin, &bracket_begin_id, &bracket_begin_time);
     parse_bracket(bracket_end, &bracket_end_id, &bracket_end_time);
 
+    if (window_id && strlen(window_id) > 20) {
+	fprintf(stderr, "window-id too long: %s.\n", window_id);
+	exit(1);
+    }
+
+    if (leave_prefix && strlen(leave_prefix) > MAXPATHLEN) {
+	fprintf(stderr, "save-dir too long: %s.\n", leave_prefix);
+	exit(1);
+    }
+	
+    if (splash_prefix && strlen(splash_prefix) > MAXPATHLEN) {
+	fprintf(stderr, "data-dir too long: %s.\n", splash_prefix);
+	exit(1);
+    }
+
+    if (proxy_user && strlen(proxy_user) > 100) {
+	fprintf(stderr, "proxy-user too long: %s.\n", proxy_user);
+	exit(1);
+    }
+
+    if (proxy_name && strlen(proxy_name) > 100) {
+	fprintf(stderr, "proxy-name too long: %s.\n", proxy_name);
+	exit(1);
+    }
+
+    if (dream_server && strlen(dream_server) > 100) {
+	fprintf(stderr, "server too long: %s.\n", dream_server);
+	exit(1);
+    }
+
     if (nice_level < -20) {
 	fprintf(stderr, "nice level must be -20 or greater, not %d.\n",
 		nice_level);
@@ -1750,20 +1750,23 @@ flags_init(int *argc, char ***argv) {
 	    fprintf(stderr, "HOME envar not defined\n");
 	    cleanup_and_exit(1);
 	}
-	  
-	sprintf(b, "%s/.sheep/", hom);
+	if (strlen(hom) > MAXPATHLEN) {
+	    fprintf(stderr, "HOME envar too long: %s.\n", hom);
+	    cleanup_and_exit(1);
+	}
+	snprintf(b, MAXBUF, "%s/.sheep/", hom);
 	leave_prefix = strdup(b);
     } else if (leave_prefix[strlen(leave_prefix)-1] != '/') {
 	char b[MAXBUF];
-	sprintf(b, "%s/", leave_prefix);
+	snprintf(b, MAXBUF, "%s/", leave_prefix);
 	leave_prefix = strdup(b);
     }
 
-    sprintf(id_file, "%sid", leave_prefix);
+    snprintf(id_file, MAXBUF, "%sid", leave_prefix);
 
     if (1) {
 	char b[MAXBUF];
-	sprintf(b, "mkdir -p %s", leave_prefix);
+	snprintf(b, MAXBUF, "mkdir -p %s", leave_prefix);
 	mysystem(b, "mkdir leave prefix");
     }
 
@@ -1799,7 +1802,7 @@ do_lock() {
 
     struct flock fl;
   
-    sprintf(fn, "%slock", leave_prefix);
+    snprintf(fn, MAXBUF, "%slock", leave_prefix);
     if (-1 == (fd = creat(fn, S_IRWXU))) {
 	perror(fn);
 	cleanup_and_exit(1);
@@ -1847,7 +1850,7 @@ void set_uniqueid() {
   }
   d[0] ^= tv.tv_sec;
   d[1] ^= tv.tv_usec;
-  sprintf(uniqueid, "%08X%08X", d[0], d[1]);
+  snprintf(uniqueid, uniqueid_len+1, "%08X%08X", d[0], d[1]);
 }
 
 int
@@ -1884,7 +1887,7 @@ main(int argc, char **argv) {
       char pbuf[MAXBUF];
       int changed = 0;
 
-      sprintf(pbuf, "%src", leave_prefix);
+      snprintf(pbuf, MAXBUF, "%src", leave_prefix);
       frc = fopen(pbuf, "rb");
       if (NULL != frc) {
 	int tlmm;
@@ -1920,7 +1923,7 @@ main(int argc, char **argv) {
 
       /* write rc file */
       if (changed) {
-	sprintf(tbuf, "%src.tmp", leave_prefix);
+	  snprintf(tbuf, MAXBUF, "%src.tmp", leave_prefix);
 	if (frc = fopen(tbuf, "wb")) {
 	  fprintf(frc, "%d\n%s\n", leave_max_megabytes, uniqueid);
 	  fclose(frc);
@@ -1934,9 +1937,21 @@ main(int argc, char **argv) {
     }
 
 
-    if (logfile) {
-      freopen(logfile, "ab", stdout);
-      freopen(logfile, "ab", stderr);
+    if (logfile && logfile[0]) {
+	if (NULL == freopen(logfile, "a", stdout)) {
+	    perror(logfile);
+	}
+	setlinebuf(stdout);
+	if (NULL == freopen(logfile, "a", stderr)) {
+	    perror(logfile);
+	}
+	setlinebuf(stderr);
+    }
+
+    if (debug) {
+	printf("=====================================\n"
+	       "electric sheep v%s\n", VERSION);
+	timestamp();
     }
 
     do_lock();
@@ -1944,19 +1959,18 @@ main(int argc, char **argv) {
     reset_fuse = reset_fuse_length;
 
     if (proxy_name) {
-	sprintf(curl_cmd, "nice -n %d curl --proxy %s",
+	snprintf(curl_cmd, MAXBUF, "nice -n %d curl --proxy %s",
 		nice_level, proxy_name);
 	if (proxy_user) {
 	  strcat(curl_cmd, " --proxy-user ");
 	  strcat(curl_cmd, proxy_user);
 	}
     } else
-	sprintf(curl_cmd, "nice -n %d curl", nice_level);
+	snprintf(curl_cmd, MAXBUF, "nice -n %d curl", nice_level);
 
     srandom(time(0));
 
     not_playing();
-    default_background(0);
 
     if (display_anim) {
         make_display_process();
